@@ -3,7 +3,7 @@
  */
 
 /* 
- * Copyright (C) 1986, 1988, 1989, 1991-2011 the Free Software Foundation, Inc.
+ * Copyright (C) 1986, 1988, 1989, 1991-2012 the Free Software Foundation, Inc.
  * 
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -24,7 +24,7 @@
  */
 
 /* FIX THIS BEFORE EVERY RELEASE: */
-#define UPDATE_YEAR	2011
+#define UPDATE_YEAR	2012
 
 #include "awk.h"
 #include "getopt.h"
@@ -148,10 +148,8 @@ int do_binary = FALSE;		/* hands off my data! */
 int do_sandbox = FALSE; 	/* sandbox mode - disable 'system' function & redirections */
 int use_lc_numeric = FALSE;	/* obey locale for decimal point */
 
-#ifdef MBS_SUPPORT
+#if MBS_SUPPORT
 int gawk_mb_cur_max;		/* MB_CUR_MAX value, see comment in main() */
-#else
-const int gawk_mb_cur_max = 1;
 #endif
 
 FILE *output_fp;		/* default output for debugger */
@@ -267,7 +265,7 @@ main(int argc, char **argv)
 	setlocale(LC_TIME, "");
 #endif
 
-#ifdef MBS_SUPPORT
+#if MBS_SUPPORT
 	/*
 	 * In glibc, MB_CUR_MAX is actually a function.  This value is
 	 * tested *a lot* in many speed-critical places in gawk. Caching
@@ -306,6 +304,8 @@ main(int argc, char **argv)
 	/* init array handling. */
 	array_init();
 
+	output_fp = stdout;
+
 	/* we do error messages ourselves on invalid options */
 	opterr = FALSE;
 
@@ -556,7 +556,7 @@ out:
 	if (do_lint && os_is_setuid())
 		warning(_("running %s setuid root may be a security problem"), myname);
 
-#ifdef MBS_SUPPORT
+#if MBS_SUPPORT
 	if (do_binary) {
 		if (do_posix)
 			warning(_("`--posix' overrides `--binary'"));
@@ -678,7 +678,6 @@ out:
 		setlocale(LC_NUMERIC, "");
 #endif
 
-	output_fp = stdout;
 	interpret(code_block);
 
 	if (do_profiling) {
@@ -924,7 +923,7 @@ static const struct varinit varinit[] = {
 {&FILENAME_node, "FILENAME",	"",	0,  NULL, NULL,	FALSE, 0 },
 {&FNR_node,	"FNR",		NULL,	0,  update_FNR, set_FNR,	TRUE, 0 },
 {&FS_node,	"FS",		" ",	0,  NULL, set_FS,	FALSE, 0 },
-{&FPAT_node,	"FPAT",		"[^[:space:]]+", 0,  NULL, set_FPAT,	FALSE, 0 },
+{&FPAT_node,	"FPAT",		"[^[:space:]]+", 0,  NULL, set_FPAT,	FALSE, NON_STANDARD },
 {&IGNORECASE_node, "IGNORECASE", NULL,	0,  NULL, set_IGNORECASE,	FALSE, NON_STANDARD },
 {&LINT_node,	"LINT",		NULL,	0,  NULL, set_LINT,	FALSE, NON_STANDARD },
 {&NF_node,	"NF",		NULL,	-1, update_NF, set_NF,	FALSE, 0 },
@@ -1127,6 +1126,26 @@ is_std_var(const char *var)
 	return FALSE;
 }
 
+
+/* get_spec_varname --- return the name of a special variable
+	with the given assign or update routine.
+*/
+
+const char *
+get_spec_varname(Func_ptr fptr)
+{
+	const struct varinit *vp;
+
+	if (! fptr)
+		return NULL;
+	for (vp = varinit; vp->name != NULL; vp++) {
+		if (vp->assign == fptr || vp->update == fptr)
+			return vp->name;
+	}
+	return NULL;
+}
+
+
 /* arg_assign --- process a command-line assignment */
 
 int
@@ -1353,17 +1372,20 @@ init_groupset()
 	 */
 	ngroups = getgroups(0, NULL);
 #endif
-	if (ngroups == -1)
-		fatal(_("could not find groups: %s"), strerror(errno));
-	else if (ngroups == 0)
+	/* If an error or no groups, just give up and get on with life. */
+	if (ngroups <= 0)
 		return;
 
 	/* fill in groups */
 	emalloc(groupset, GETGROUPS_T *, ngroups * sizeof(GETGROUPS_T), "init_groupset");
 
 	ngroups = getgroups(ngroups, groupset);
-	if (ngroups == -1)
-		fatal(_("could not find groups: %s"), strerror(errno));
+	/* same thing here, give up but keep going */
+	if (ngroups == -1) {
+		efree(groupset);
+		ngroups = 0;
+		groupset = NULL;
+	}
 #endif
 }
 
